package de.unima.ki.pi2.ptest.aufgabe2.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.HashSet;

import org.junit.Before;
import org.junit.Test;

import de.unima.ki.pi2.ptest.aufgabe2.api.CharCounterX;
import de.unima.ki.pi2.ptest.aufgabe2.impl.CharCounterXImpl;


public class CharCounterXTest {

	private CharCounterX counterx;
	
	@Before
	public void init(){ 
		this.counterx = new CharCounterXImpl();
	}

	@Test
	public void testSimple1() {
		String text = "aaaa bbb cc d";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		assertEquals('d', c1);
		assertEquals('c', c2);
	}

	
	@Test
	public void testSimple2() {
		String text = "abcdefg abcdefg abcdefg xx abcdefg yyy abcdefg abcdefg abcdefg abcdefg";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		assertEquals('x', c1);
		assertEquals('y', c2);
	}
	
	@Test
	public void testNotEnough1() {
		String text = "";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		assertNull(cc);
	}
	
	@Test
	public void testNotEnough2() {
		String text = "a";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		assertNull(cc);
	}
	
	@Test
	public void testNotEnough3() {
		String text = "qqqqqqqqqqqqqqqqqqqqqqqqqq";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		assertNull(cc);
	}	
	
	
	@Test
	// mehrere buchstaben haben dieselbe haeufigkeit, verschiedene rueckgabewerte erlaubt
	public void test1() {
		String text = "a otto otto otto b";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		// moegliche ergebnis kandidaten 
		HashSet<Character> c1Cand = new HashSet<Character>();
		c1Cand.add('a');
		c1Cand.add('b');	
		HashSet<Character> c2Cand = new HashSet<Character>();
		c2Cand.add('a');
		c2Cand.add('b');		
		assertTrue("niedrigster buchstabe falsch", c1Cand.contains(c1));
		assertTrue("zweitniedrigster buchstabe falsch",c2Cand.contains(c2));
		// die rueckgabewerte muessen verschieden sein
		assertTrue("buchstaben muessen verschieden sein", c1 != c2);
	}
	
	@Test
	// mehrere buchstaben haben dieselbe haeufigkeit, verschiedene rueckgabewerte erlaubt
	public void test2() {
		String text = "a horse is a horse of course of course";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		// moegliche ergebnis kandidaten 
		HashSet<Character> c1Cand = new HashSet<Character>();
		c1Cand.add('i');
		HashSet<Character> c2Cand = new HashSet<Character>();
		c2Cand.add('c');
		c2Cand.add('a');
		c2Cand.add('h');
		c2Cand.add('f');
		c2Cand.add('u');
		assertTrue("niedrigster buchstabe falsch", c1Cand.contains(c1));
		assertTrue("zweitniedrigster buchstabe falsch",c2Cand.contains(c2));
		// die rueckgabewerte muessen verschieden sein
		assertTrue("buchstaben muessen verschieden sein", c1 != c2);
	}
	
	@Test
	// mehrere buchstaben haben dieselbe haeufigkeit, verschiedene rueckgabewerte erlaubt
	public void test3() {
		String text = "aaaaxxxbbbbxxxccccxxxddddxxxeeeeee";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		// moegliche ergebnis kandidaten 
		HashSet<Character> c1Cand = new HashSet<Character>();
		c1Cand.add('a');
		c1Cand.add('b');
		c1Cand.add('c');
		c1Cand.add('d');
		HashSet<Character> c2Cand = new HashSet<Character>();
		c2Cand.add('a');
		c2Cand.add('b');
		c2Cand.add('c');
		c2Cand.add('d');
		assertTrue("niedrigster buchstabe falsch", c1Cand.contains(c1));
		assertTrue("zweitniedrigster buchstabe falsch",c2Cand.contains(c2));
		// die rueckgabewerte muessen verschieden sein
		assertTrue("buchstaben muessen verschieden sein", c1 != c2);
	}	
	
	@Test
	// mehrere buchstaben haben dieselbe haeufigkeit, verschiedene rueckgabewerte erlaubt
	public void test4() {
		String text = "lorem ipsum dolor sit amet consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat sed diam voluptua at vero eos et accusam et iusto duo dolores et ea rebum";
		this.counterx.parse(text);
		char[] cc = this.counterx.getRareChars();
		char c1 = cc[0];
		char c2 = cc[1];
		// moegliche ergebnis kandidaten 
		HashSet<Character> c1Cand = new HashSet<Character>();
		c1Cand.add('q');
		HashSet<Character> c2Cand = new HashSet<Character>();
		c2Cand.add('y');
		c2Cand.add('b');
		c2Cand.add('g');
		assertTrue("niedrigster buchstabe falsch", c1Cand.contains(c1));
		assertTrue("zweitniedrigster buchstabe falsch",c2Cand.contains(c2));
		// die rueckgabewerte muessen verschieden sein
		assertTrue("buchstaben muessen verschieden sein", c1 != c2);
	}	
	
	
}
